function renderData() {
	renderFoods(elements.foods, elements.foodPages);
	renderCategories(elements.categories);
}

function renderCategories(container) {
	for (var key in data.categories) {
		var item = element('a', {
			'href': '#category:' + key,
			'data-weight': data.categories[key]['weight']
		}, null, null, false);
		if (data.categories[key].icon) {
			item.style.backgroundImage = "url(" + data.categories[key].icon + ")";
		}
		setTranslationToElement(item, data.categories[key].title);
		container.appendChild(item);
	}
}

function renderFoods(listContainer, pageContainer) {
	var categoryCount = {};
	removeAllChildren(listContainer);
	var language = getLanguage();
	if (!data.foods) {
		return;
	}
	for (var i = 0, id = 0; i < data.foods.length; i++) {
		if (data.foods[i].title[getLanguage()]) {
			for (var j in data.foods[i].title[getLanguage()]) {
				var c = data.foods[i].category;
				//console.log(c);
				if (categoryCount[c] === undefined) {
					categoryCount[c] = 0;
				}
				if (categoryCount[c] !== undefined) {
					categoryCount[c]++;
				}
				if (!config.light || (config.light && categoryCount[c] < 11)) {
					var dangerousProperties = getDangerousProperties(data.foods[i].properties);
					var warningChar = (hasHistamineIntolerance() && dangerousProperties.length > 0) ? '!' : '';
					var level = getLevelFromPropertiesAndIntolerances(data.foods[i].properties);
					var textLevel = (level * 1) + '';
					if (textLevel == 0) {
						textLevel = '?';
					} else {
						textLevel = (textLevel * 1 - 1) + '';
					}
					if (getLevels().indexOf(level) > -1) {
						var title = data.foods[i].title[language][j], category = data.foods[i].category, href = '#food:' + id;
						// Item
						var item = element('a', {
							'': title,
							'href': href,
							'data-category': category,
							'data-id': id,
							'data-level': level,
							'data-rating': getCustomLevelRating(id)
						}, [
							element('div', {'class': 'level level-' + level}, [
								element('div', {
									'data-id': id,
									'data-level': level,
									'class': 'custom-level' + getCustomLevelClass(id),
								}, [
									element('div', {'class': 'level-inlay'}, [
										element('span', {'class': 'text-level', '': textLevel}),
										element('span', {'class': 'real-level', '': '(' + textLevel + ')'}),
										element('span', {
											'class': 'food-warning',
											'': warningChar
										})
									])
								])
							])
						]);
						listContainer.appendChild(item);
						item = element('a', {
							'data-id': id,
							'': title,
							'href': href,
							'data-category': category,
							//'class': 'inactive'
						}, [
							element('div', {'class': 'level level-' + level}, [
								element('div', {
									'data-id': id,
									'data-level': level,
									'class': 'custom-level' + getCustomLevelClass(id)
								}, [
									element('div', {'class': 'level-inlay'}, [
										element('span', {'class': 'text-level', '': textLevel}),
										element('span', {'class': 'real-level', '': '(' + textLevel + ')'}),
										element('span', {'class': 'food-warning', '': warningChar})

									])
								])
							])
						]);
						elements.searchFoodPages.appendChild(item);

						// Page
						var page = element('div', {'data-role': 'page', 'id': 'food:' + id, 'data-id': id}, [
							element('header', null, [
								element('a', {'class': 'left'}, [
									element('i', {'class': 'fa fa-angle-left'}, null)
								], {'onclick': back}),
								element('h1', {'': 'Details'})
							]),
							element('section', null, [
								element('div', {'class': 'headline'}, [
									element('span', {'': title})
								]),
								element('div', {'class': 'level level-' + level}, [
									element('div', {
										'data-id': id,
										'data-level': level,
										'class': 'custom-level' + getCustomLevelClass(id)
									}, [
										element('div', {'class': 'level-inlay'}, [
											element('span', {'class': 'text-level', '': textLevel}),
											element('span', {'class': 'real-level', '': '(' + textLevel + ')'}),
											element('span', {'class': 'food-warning food-warning-a', '': warningChar})
										])
									])
								])
							].concat(createFoodPageProperties(data.foods[i].properties, dangerousProperties).concat([createCustomRating(id)])))
						]);
						pageContainer.appendChild(page);
					}
					id++;
				}
			}
		}
	}
	getPages();
}

function createCustomRating(id) {
	return element('div', {'class': 'spacing custom-rating-container'}, [
		element('h3', {
			'data-translatable': '',
			'data-translation-de': 'Meine Wertung',
			'data-translation-en': 'My rating'
		}),
		element('div', {'class': 'my-ratings'}, [
			createCustomLevelToggle(id, 1, '0'),
			createCustomLevelToggle(id, 2, 1),
			createCustomLevelToggle(id, 3, 2),
			createCustomLevelToggle(id, 4, 3)
		])
	]);
}

function createCustomLevelToggle(id, level, textLevel) {
	var ratings = getRatings();
	var e = element('div', {
		'class': 'level level-' + level,
		'data-food-id': id,
		'data-food-level': textLevel
	}, [
		element('div', {'class': 'text-level-caption', '': textLevel}),
		element('i', {'class': 'fa fa-times'})
	], {'onclick': toggleCustomRating});
	if (!isNaN(ratings[id])) {
		ratings[id] *= 1;
		if (ratings[id] && ratings[id] + 1 == level) {
			e.setAttribute('data-active', '');
		}
	}
	return e;
}

function createFoodPageProperties(properties, dangerousProperties) {
	var elements = [];
	var order = ['histamine', 'lactose', 'gluten', 'fructose'];
	var language = getLanguage();
	//for (var key in properties) {
	for (var orderKey in order) {
		var key = order[orderKey];
		var level = (properties[key].value * 1);
		if (isNaN(level) || level == '-') {
			level = 0;
		}
		var textLevel = (level * 1) + '';
		if (textLevel == 0) {
			textLevel = '?';
		} else {
			textLevel = (textLevel * 1 - 1) + '';
		}
		var description = properties[key].description ? properties[key].description[language] : '';
		var title = data.properties[key] ? data.properties[key].title[language] : '';
		var caption = {};
		//getValue()

		var captionText = (hasHistamineIntolerance() && dangerousProperties[orderKey]) ? '!' : '';
		/*
		 if (captionText == '!') {
		 description = '! = <span data-translatable data-translation-de="schnell verderblich, rasche Histaminbildung" data-translation-en="Quickly digestible, fast formation of histamine"></span>' + description;
		 }
		 */

		if (captionText == '!') {
			for (var aKey in properties[key].attributes) {
				if (properties[key].attributes[aKey].title == 0) {
					properties[key].attributes[aKey].warning = '! = <span data-translatable data-translation-de="schnell verderblich,<br>rasche Histaminbildung" data-translation-en="Quickly digestible, fast formation of histamine"></span>';
					break;
				}
			}
		}

		//properties['histamine'].attributes[];

		elements.push(element('div', {'class': 'property'}, [
			element('header', null, [
				element('h2', {'': title}),
				element('div', {'class': 'level level-' + level, '': (caption.caption ? caption.caption : '')}, [
					element('div', {
						'class': 'custom-level'
					}, [
						element('div', {'class': 'level-inlay'}, [
							element('span', {'class': 'text-level', '': textLevel}),
							element('span', {
								'class': 'food-warning',
								'_': captionText
							})
						])
					])
				])
			]),
			element('section', null, [
				element('div', {'_': description, 'class': 'description'}, null, null, true),
				element('div', {'class': 'attributes'}, createFoodPagePropertyAttributes(properties[key].attributes, key), null, true)
			], null, true)
		]));
	}
	return elements;
}

function createFoodPagePropertyAttributes(attributes, propKey) {
	var elements = [];
	var language = getLanguage();
	for (var key in attributes) {
		var title = data.attributes[attributes[key].title][language];
		var value;
		var caption = {};
		if (attributes[key].warning) {
			caption.value = attributes[key].warning;
		} else {
			value = (typeof attributes[key].value == 'object' ? attributes[key].value[language] : attributes[key].value);
			getValue(value, caption);
		}
		elements.push(element('div', {'class': 'attribute'}, [
			element('div', {'': title, 'class': 'title'}),
			element('div', {'_': caption.value, 'class': 'value'}),
		]));
	}
	//if (propKey == 'fructose' && attributes && attributes.length > 0) {
	if (propKey == 'fructose' && attributes) {
		elements.push(element('div', {
			'class': 'fru-notice',
			'data-translatable': '',
			'data-translation-de': '(g/100g)',
			'data-translation-en': '(g/100g)'
		}));
	}
	return elements;
}

function isNumber(n) {
	'use strict';
	if (!isNaN(n)) {
		return true;
	}
	if (n.replace) {
		n = n.replace(/\./g, '').replace(',', '.');
		return !isNaN(parseFloat(n)) && isFinite(n);
	}
}

function getDangerousProperties(properties) {
	var dang = [];
	for (var propKey in properties) {
		if (hasDangerousAttribute(properties[propKey])) {
			dang.push(propKey);
		}
	}
	return dang;
}

function hasDangerousAttribute(property) {
	if (property && property.attributes) {
		for (var attrKey in property.attributes) {
			var v = {};
			getValue(property.attributes[attrKey].value, v);
			if (v.caption == '!') {
				return true;
			}
		}
	}
}

function getValue(string, result) {
	if (isNumber(string)) {
		result.value = string;
	} else {
		if (string.substring && string.substring(string.length - 1) == '!') {
			result.caption = '!';
			string = string.substring(0, string.length - 1);
		}
		if (string) {
			if (string == '?') {
				result.value = '?';
			} else if (string == 'H' || string == 'L' || string == 'A' || string == 'B') {
				//result.value = 'ja';
				result.value = '<span data-translatable data-translation-en="yes" data-translation-de="ja"></span>';
			} else if (string == '') {
				result.value = '-';
			} else {
				result.value = string;
			}
		} else {
			result.value = '-';
		}
	}
	if (!result.value) {
		result.value = '-';
	}
	return result;
}

function renderCustomRatings() {
	var ratings = getRatings();
	for (var id in ratings) {
		var c = ratings[id];
		setCustomLevelToFoodsWithId(id, c);
		var reals = document.querySelectorAll('*[data-id="' + id + '"] .real-level');
		for (var i = 0; i < reals.length; i++) {
			reals[i].style.display = 'inline';
		}

		var texts = document.querySelectorAll('*[data-id="' + id + '"] .text-level');
		for (i = 0; i < texts.length; i++) {
			texts[i].textContent = c;
			//texts[i].innerText = c;
		}
	}
}

function hideRealLevels() {
	var elements = document.querySelectorAll('.real-level');
	for (var i = 0; i < elements.length; i++) {
		elements[i].style.display = 'none';
	}
}